################################################################################
# Utilities for DNA methylation analysis of blood progenitors
#-------------------------------------------------------------------------------
# author: Fabian Müller
# date:   2016-09-13
################################################################################

################################################################################
# Annotation (Cell type)
################################################################################
# preferenctial order for cell types in plotting, etc.
CELL.TYPE.ORDER <- c("HSC", "MPP", "CMP", "MEP", "GMP", "GMP10p", "CLP", "MLP0", "MLP1", "MLP2", "MLP3", "Mono", "Neut", "NK", "Bcell", "CD4", "CD8", "MK")
getOrderedCellTypeFactor <- function(cellTypeVec){
	lvls <- sort(unique(cellTypeVec))
	orderedLvls <- c(intersect(CELL.TYPE.ORDER, lvls), setdiff(lvls, CELL.TYPE.ORDER))
	return(factor(cellTypeVec, levels=orderedLvls))
}
################################################################################
# missing data handling
################################################################################
handleMissingData <- function(x, method="na.rm"){
	res <- list(data=x, removed=c(), retained=1:nrow(x))
	if (method=="na.rm"){
		logger.info(c("Discarding NA values"))
		rows.nao <- rowSums(is.na(x)) < 1
		num.nao <- sum(rows.nao)
		if (num.nao < nrow(mm))	logger.info(c("Retained", num.nao, "of", nrow(mm), "measurements"))
		res[["data"]] <- x[rows.nao, , drop=FALSE]
		res[["retained"]] <- which(rows.nao)
		res[["removed"]]  <- which(!rows.nao)
	} else if (is.element(method, c("impute.knn1", "impute.knn5", "impute.knn10"))){
		require(impute)
		k <- as.numeric(gsub("^impute\\.knn(.*)$", "\\1",method))
		logger.info(c("Employing ", k, "-nearest-neighbor imputation"))
		x.imp <- impute.knn(x, k=k, colmax=0.98)
		res[["data"]] <- x.imp$data
	} else {
		stop("Unknown method for handling missing data")
	}
	return(res)
}
